// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "OrderProperties.h"
#include "OrderPropertiesDlg.h"

STDMETHODIMP COrderPropertiesDlg::OnDataError(
                                 LPDISPATCH /*obj*/,
                                 BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2T(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP COrderPropertiesDlg::OnGWConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   m_lblGWConnection.SetWindowText(_T("GW connection : ") + ConnectionStatusToString(newStatus));

   if (newStatus == csConnectionUp)
   {
      m_spCQGCEL->put_AccountSubscriptionLevel(aslAccountUpdatesAndOrders);
   }
   else
   {
      // Disable query buttons
      EnableControls(false);
      
      // Cancel current query
      OnBnClickedBtnQueryCancel();
   }

   SetGWLogonButtonStatus(newStatus);

   return S_OK;
}

STDMETHODIMP COrderPropertiesDlg::OnDataConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   m_lblDataConnection.SetWindowText(_T("Data connection : ") + ConnectionStatusToString(newStatus));

   return S_OK;
}

STDMETHODIMP COrderPropertiesDlg::OnAccountChanged(
                                 eAccountChangeType change, 
                                 ICQGAccount* /*account*/, 
                                 ICQGPosition* /*position*/)
{
   try
   {
      if (change == actAccountsReloaded)
      {
         ReloadAccounts();
         EnableControls(true);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP COrderPropertiesDlg::OnInstrumentResolved(
   BSTR /*symbol*/, ICQGInstrument* /*cqg_instrument*/, ICQGError* /*cqg_error*/)
{
   try
   {
      if (m_spSelectedOrder != NULL)
      {
         // Clear old data
         ClearProperties();
         ClearFills();

         // Update properties for selected order
         ShowProperties(m_spSelectedOrder);

         // Update selected order fills
         ShowFills(m_spSelectedOrder);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP COrderPropertiesDlg::OnQueryProgress(
                                 ICQGOrdersQuery* query, 
                                 ICQGError* /*error*/)
{
   try
   {
      eRequestStatus reqStatus;
      HRESULT hr = query->get_Status(&reqStatus);
      AssertCOMError<ICQGOrdersQuery>(query, hr);

      ShowQueryStatus(query);
      
      if (reqStatus == rsSuccess)
      {
         ATL::CComPtr<ICQGOrders> spOrders;
         hr = query->get_Orders(&spOrders);
         AssertCOMError<ICQGOrdersQuery>(query, hr);

         ShowOrders(spOrders);
      }

      if (reqStatus != rsInProgress)
      {
         m_IsInQuery = false;
         EnableControls(true);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}
