// OrdersFilterParametersDlg.h : header file
//

#pragma once

class COrdersFilterParametersDlg;

/**
\typedef ATL::IDispEventImpl<1, COrdersFilterParametersDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImplOF
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              COrdersFilterParametersDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImplOF;

/// \brief COrdersFilterParametersDlg dialog. 
/// \brief Used to select the parameter used to filter orders.
/// \brief This class advises to CQGCEL events to fill auto-subscribed instruments
/// \brief to the combo box.
class COrdersFilterParametersDlg : public CDialog, public ICQGCELDispEventImplOF
{
public:
   /// \brief Constructor
   /// \param spCel - CQGCEL instance
   /// \param pParent - Parent window
	COrdersFilterParametersDlg(
                        const ATL::CComPtr<ICQGCEL>& spCEL,
                        CWnd* pParent = NULL);

   /// \enum Dialog Data
	enum { IDD = IDD_ORDERS_FILTER_DIALOG };

   /// \brief Enables/disables corresponding combo boxes.
   afx_msg void OnBnClickedRdInstrument();

   /// \brief Enables/disables corresponding combo boxes.
   afx_msg void OnBnClickedRdType();

   /// \brief Enables/disables corresponding combo boxes.
   afx_msg void OnBnClickedRdStatus();

   /// \brief Enables/disables corresponding combo boxes.
   afx_msg void OnBnClickedRdSide();

   /// \brief Used to unadvise from CQGCEL events on dialog destroy.
   afx_msg void OnDestroy();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

protected:

   /// \brief Generated message map functions
   virtual void DoDataExchange(
                        CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the application has set the input 
   /// \return focus to one of the controls in the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter.
   virtual void OnOK();

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(COrdersFilterParametersDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
   END_SINK_MAP()

protected:

   /// \brief Fired when new instrument is resolved and subscribed or failure happened during a resolving process.
   /// \param symbol - Requested symbol
   /// \param instrument - Subscribed instrument object
   /// \param error - Error occured during a symbol resolution process.
   /// \return S_OK
   STDMETHOD(OnInstrumentResolved)(BSTR symbol, ICQGInstrument* instrument, ICQGError* cqg_error);

private:
   
   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes instruments combo box.
   void InitializeInstrumentsCombo();

   /// \brief Initializes sides combo box.
   void InitializeSidesCombo();

   /// \brief Initializes statuses combo box.
   void InitializeFilterStatusesCombo();

   /// \brief Initializes order's types combo box.
   void InitializeTypesCombo();

   /// \brief Unadvises from CQGCEL events.
   void UnadviseFromCQGCEL();

public:

   /// \enum eFilterType. Filter types.
   enum eFilterType;

   /// \brief Returns filter type.
   /// \return filter type
   eFilterType GetFilterType() const;

   /// \brief Returns instrument full name filter.
   /// \return instrument full name
   CString GetFullName() const;

   /// \brief Returns order type filter.
   /// \return order type
   eOrderType GetOrderType() const;

   /// \brief Returns order status filter.
   /// \return order status
   eOrderStatus GetOrderStatus() const;

   /// \brief Returns order side filter.
   /// \return order side
   eOrderSide GetOrderSide() const;

private:

   /// \brief CQGCEL object
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Filter type.
   eFilterType m_FilterType;

   /// \brief Instrument full name filter.
   CString m_FullName;

   /// \brief Order type filter.
   eOrderType m_OrderType;

   /// \brief Order status filter.
   eOrderStatus m_OrderStatus;

   /// \brief Order side filter.
   eOrderSide m_OrderSide;

private:

   /// \brief Order statuses combo box control.
   CComboBox m_cmbStatuses;

   /// \brief Order sides combo box control.
   CComboBox m_cmbSides;

   /// \brief Order types combo box control.
   CComboBox m_cmbTypes;

   /// \brief Instrument names combo box control.
   CComboBox m_cmbInstruments;

   /// \brief Instrument radio button control.
   CButton m_rdInstrument;

   /// \brief Type radio button control.
   CButton m_rdType;

   /// \brief Status radio button control.
   CButton m_rdStatus;

   /// \brief Side radio button control.
   CButton m_rdSide;

   /// \brief OK button control.
   CButton m_btnOk;
};

enum COrdersFilterParametersDlg::eFilterType
{
   ftInstrument,
   ftType,
   ftStatus,
   ftSide
};
